package com.sztzjy.money_management.controller;

import com.sztzjy.money_management.annotation.AnonymousAccess;
import com.sztzjy.money_management.config.Constant;
import com.sztzjy.money_management.entity.Chapter;
import com.sztzjy.money_management.entity.ChapterExample;
import com.sztzjy.money_management.mapper.ChapterMapper;
import com.sztzjy.money_management.service.ChapterService;
import com.sztzjy.money_management.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@RestController
@Api(tags = "章节相关相关")
@RequestMapping("api/ChapterController")
public class ChapterController {
    @Autowired
    ChapterMapper chapterMapper;
    @Autowired
    ChapterService chapterService;

    //章节添加
    @AnonymousAccess
    @ApiOperation("章节添加")
    @PostMapping("chapterAdd")
    public ResultEntity chapterAdd(@RequestParam String chapterName,
                                   @RequestParam String schoolId,
                                   @ApiParam("科 0代表知识概要 1代表学习资源 2代表题库中心") @RequestParam String familes) {
        if(StringUtils.isBlank(schoolId)){
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "schoolId不能为空");
        }
        Boolean flag=chapterService.chapterAdd(chapterName,schoolId,familes);
        if(flag){
            return new ResultEntity<>(HttpStatus.OK, "添加章节成功");
        }else {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "添加章节失败");
        }

    }

    //章节查询
    @AnonymousAccess
    @ApiOperation("章节查询")
    @PostMapping("chapterSelect")
    public ResultEntity<List<Chapter>> chapterSelect(@RequestParam String schoolId,@ApiParam("科 0代表知识概要 1代表学习资源 2代表题库中心") @RequestParam String familes) {
        if(StringUtils.isBlank(schoolId)){
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "schoolId不能为空");
        }
        List<Chapter> list=chapterService.chapterSelect(schoolId,familes);
        for (int i = 0; i < list.size(); i++) {
            if(list.get(i).getChapterName().equals("财务分析")){
                list.get(i).setChapterName("家庭财务报表的编制与分析");
            }
        }
        return new ResultEntity<>(HttpStatus.OK, "章节查询成功",list);
    }


    //客户案例左侧章节节查询
    @AnonymousAccess
    @ApiOperation("客户案例左侧章节节查询")
    @PostMapping("chapterSelectInner")
    public ResultEntity<List<Chapter>> chapterSelectInner() {
        ChapterExample chapterExample = new ChapterExample();
        ChapterExample.Criteria criteria = chapterExample.createCriteria();
        criteria.andSourceEqualTo(Constant.BUILT_IN_SCHOOL_ID);
        chapterExample.setOrderByClause("add_time asc");
        List<Chapter> chapters = chapterMapper.selectByExample(chapterExample);
        return new ResultEntity<>(HttpStatus.OK, "章节查询成功",chapters);
    }

    //章节删除
    @AnonymousAccess
    @ApiOperation("章节删除")
    @PostMapping("chapterDelete")
    public ResultEntity<List<Chapter>> chapterDelete( @RequestParam String chapterId,
                                                      @RequestParam String schoolId) {
        if(StringUtils.isBlank(schoolId)){
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "schoolId不能为空");
        }
        Boolean flag=chapterService.chapterDelete(chapterId,schoolId);
        if(flag){
            return new ResultEntity<>(HttpStatus.OK, "章节删除成功");
        }else {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "章节删除失败");
        }
    }

}
