package com.sztzjy.money_management.controller;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.money_management.annotation.AnonymousAccess;
import com.sztzjy.money_management.entity.ObjectiveQuestionWithBLOBs;
import com.sztzjy.money_management.entity.dto.ObjectiveQuestionDto;
import com.sztzjy.money_management.entity.dto.TopicDto;
import com.sztzjy.money_management.mapper.ObjectiveQuestionMapper;
import com.sztzjy.money_management.service.ObjectiveQuestionService;
import com.sztzjy.money_management.util.ResultEntity;
import com.sztzjy.money_management.util.file.IFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author xcj
 * @Date 2024/8/12
 */

@RestController
@Api(tags = "题库管理")
@RequestMapping("api/topic")
public class TopicController {
    @Autowired
    private ObjectiveQuestionMapper objectiveQuestionMapper;
    @Autowired
    private ObjectiveQuestionService objectiveQuestionService;
    @Autowired
    private IFileUtil fileUtil;

    @PostMapping("/insertSingleTopic")
    @ApiOperation("新增单个客观题")
    @AnonymousAccess
    private ResultEntity<HttpStatus> insertSingleTopic(@RequestBody ObjectiveQuestionWithBLOBs objectiveQuestion) {
        if (StringUtils.isBlank(objectiveQuestion.getContent())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "题干不能为空!");
        }
        if (StringUtils.isBlank(objectiveQuestion.getAnswer())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "答案不能为空!");
        }
        if (objectiveQuestion.getScore() == null) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "分数不能为空!");
        }
        if (null == objectiveQuestion.getType()) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "题型不能为空!");
        }
        if (StringUtils.isBlank(objectiveQuestion.getOutline())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "所属大纲不能为空!");
        }
        if (!objectiveQuestion.getOutline().equals("普通题库") && StringUtils.isBlank(objectiveQuestion.getSubclass())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "大纲子类不能为空!");
        }
//        if (StringUtils.isBlank(objectiveQuestion.getChapterId()) || StringUtils.isBlank(objectiveQuestion.getChapterName())) {
//            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "所属章节信息不能为空！");
//        }
        if (objectiveQuestion.getType() == 1 || objectiveQuestion.getType() == 0) {
            if (StringUtils.isBlank(objectiveQuestion.getQuestionA()) || StringUtils.isBlank(objectiveQuestion.getQuestionB())
                    || StringUtils.isBlank(objectiveQuestion.getQuestionC()) || StringUtils.isBlank(objectiveQuestion.getQuestionD()))
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "选择题至少有四个选项!");
        }
        if (objectiveQuestion.getType() == 2) {
            if (StringUtils.isBlank(objectiveQuestion.getQuestionA()) || StringUtils.isBlank(objectiveQuestion.getQuestionB()))
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "判断题必须有两个选项!");
        }
        if (StringUtils.isBlank(objectiveQuestion.getSource())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "题目来源为999999999/学校ID!");
        }
        String answer = objectiveQuestion.getAnswer();
        if (answer.equals("A")||answer.equals("对")) {
            objectiveQuestion.setAnswer("正确");
        }
        if (answer.equals("B")||answer.equals("错")) {
            objectiveQuestion.setAnswer("错误");
        }
        objectiveQuestion.setObjectiveId(IdUtil.randomUUID());
        objectiveQuestionMapper.insert(objectiveQuestion);
        return new ResultEntity<HttpStatus>(HttpStatus.OK, "新增成功!");
    }


    @AnonymousAccess
    @ApiOperation("本地excel导入试题")
    @PostMapping("importTopicByLocal")
    private ResultEntity<HttpStatus> importTopicByLocal(@RequestParam MultipartFile file,
                                                        @ApiParam("学校ID/999999999") @RequestParam String source,
                                                        @RequestParam(required = false) String chapterID,
                                                        @RequestParam(required = false) String chapterName,
                                                        @ApiParam("普通题库/金融智能/银行从业") @RequestParam String outLine,
                                                        @RequestParam(required = false) String subClass) throws Exception {
        if (file == null) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "导入文件不能为空");
        }
        return objectiveQuestionService.insertObjectiveByExcel(file, source, chapterID, chapterName, outLine, subClass);
    }


    @PostMapping("/updateTopicById")
    @ApiOperation("编辑客观题")
    @AnonymousAccess
    private ResultEntity<HttpStatus> updateTopicById(@RequestBody ObjectiveQuestionWithBLOBs objectiveQuestionWithBLOBs) {
        if (StringUtils.isBlank(objectiveQuestionWithBLOBs.getObjectiveId())) {
            System.out.println("缺少主键ID，直接返回");
            return null;
        }
        objectiveQuestionMapper.updateByPrimaryKeySelective(objectiveQuestionWithBLOBs);
        return new ResultEntity<HttpStatus>(HttpStatus.OK, "编辑成功！");
    }


    @PostMapping("/deleteObjectiveBYList")
    @ApiOperation("客观题删除")
    @AnonymousAccess
    private ResultEntity<HttpStatus> deleteObjectiveBYList(@RequestBody List<String> objectiveQuestionId) {
        int i = objectiveQuestionMapper.batchDelete(objectiveQuestionId);
        if (i > 0) {
            return new ResultEntity<HttpStatus>(HttpStatus.OK, "删除成功！");
        }
        return new ResultEntity<HttpStatus>(HttpStatus.INTERNAL_SERVER_ERROR, "服务器异常，请联系管理员！");
    }


    @PostMapping("/getObjectiveBYId")
    @ApiOperation("客观题单个查看")
    @AnonymousAccess
    private ResultEntity<ObjectiveQuestionWithBLOBs> getObjectiveBYId(@RequestParam String objectiveQuestionId) {
        return new ResultEntity<ObjectiveQuestionWithBLOBs>(objectiveQuestionMapper.selectByPrimaryKey(objectiveQuestionId));
    }

    @PostMapping("/getObjectiveCountByChapterId")
    @ApiOperation("根据章节查看题目类型数量/模拟认证题库通用")
    @AnonymousAccess
    private ResultEntity<List<TopicDto>> getObjectiveCountByChapterId(@ApiParam("金融智能/银行从业") @RequestParam String outLine) {
        List<TopicDto> list = new ArrayList<>();
        if (outLine.equals("金融智能")) {
            List<TopicDto> list1 = objectiveQuestionMapper.getObjectiveCountByChapterId("金融智能", "理论知识");
            List<TopicDto> list2 = objectiveQuestionMapper.getObjectiveCountByChapterId("金融智能", "专业能力");
            list.addAll(list1);
            list.addAll(list2);
        }
        if (outLine.equals("银行从业")) {
            List<TopicDto> list1 = objectiveQuestionMapper.getObjectiveCountByChapterId("银行从业", "公共基础");
            List<TopicDto> list2 = objectiveQuestionMapper.getObjectiveCountByChapterId("银行从业", "个人理财");
            List<TopicDto> list3 = objectiveQuestionMapper.getObjectiveCountByChapterId("银行从业", "风险管理");
            List<TopicDto> list4 = objectiveQuestionMapper.getObjectiveCountByChapterId("银行从业", "公司信贷");
            List<TopicDto> list5 = objectiveQuestionMapper.getObjectiveCountByChapterId("银行从业", "个人贷款");
            list.addAll(list1);
            list.addAll(list2);
            list.addAll(list3);
            list.addAll(list4);
            list.addAll(list5);
        }
        return new ResultEntity<>(list);
    }


    @PostMapping("/getObjectiveByType")
    @ApiOperation("根据类型查看题目列表")
    @AnonymousAccess
    private ResultEntity<PageInfo<ObjectiveQuestionWithBLOBs>> getObjectiveByType(@RequestParam Integer index,
                                                                                  @RequestParam Integer size,
                                                                                  @ApiParam("0单选，1多选，2判断") @RequestParam(required = false) Integer type,
                                                                                  @ApiParam("普通题库/金融智能/银行从业") @RequestParam String outLine,
                                                                                  @ApiParam("理论知识/专业能力/公共基础/个人理财/风险管理/公司信贷/个人贷款") @RequestParam(required = false) String subClass,
                                                                                  @RequestParam(required = false) String content,
                                                                                  @RequestParam(required = false) String schoolId,
                                                                                  @RequestParam(required = false) String chapterId) {
        PageHelper.startPage(index, size);
        List<ObjectiveQuestionWithBLOBs> list = objectiveQuestionMapper.getObjectiveByType(type, outLine, subClass, content, schoolId, chapterId);
        PageInfo pageInfo = new PageInfo(list);
        return new ResultEntity<PageInfo<ObjectiveQuestionWithBLOBs>>(pageInfo);
    }

    @PostMapping("/commitObjectiveAnswer")
    @ApiOperation("学习测评/提交")
    @AnonymousAccess
    private void commitObjectiveAnswer(@RequestBody List<ObjectiveQuestionDto> dtoList,
                                       @RequestParam String userId,
                                       @RequestParam String chapterId) {
        objectiveQuestionService.commitObjectiveAnswer(dtoList, userId, chapterId);
    }

    @PostMapping("/getObjectiveRecord")
    @ApiOperation("学习测评/随机10条展示/回显")
    @AnonymousAccess
    private ResultEntity<List<ObjectiveQuestionDto>> getObjectiveRecord(@RequestParam String chapterId,
                                                                        @RequestParam String trainingId,
                                                                        @ApiParam("重新作答按钮传true") @RequestParam boolean flag) {
        List<ObjectiveQuestionDto> list = objectiveQuestionService.getObjectiveRecord(chapterId, trainingId, flag);
        return new ResultEntity<List<ObjectiveQuestionDto>>(list);
    }

    //    @PostMapping("/getObjectiveByRandom")
//    @ApiOperation("考试/返回指定类型和数量的随机客观题")
//    @AnonymousAccess
//    private ResultEntity<List<ObjectiveQuestionWithBLOBs>> getObjectiveByRandom(@ApiParam("题目类型 0单选，1多选，2判断）") @RequestParam int type,
//                                                                       @RequestParam int count,
//                                                                       @RequestParam String schoolId) {
//        List<ObjectiveQuestionWithBLOBs> list = objectiveQuestionMapper.getObjectiveByRandom(type, count, schoolId);
//        return new ResultEntity<>(list);
//    }
//客观题导入模板下载
    @GetMapping("downloadResource")
    @ApiOperation("客观题导入模板下载")
    @AnonymousAccess
    public void downloadResource(HttpServletResponse response) {
        fileUtil.download(response, "客观题导入模板.xlsx", "/客观题导入模板.xlsx");
    }

}
