package com.sztzjy.money_management.service.impl;

import com.sztzjy.money_management.entity.*;
import com.sztzjy.money_management.entity.dto.StuSynthesisPlanScoreAndAvgScoreDto;
import com.sztzjy.money_management.mapper.*;
import com.sztzjy.money_management.service.ScoreRankService;
import com.sztzjy.money_management.service.TrainingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ScoreRankServiceImpl implements ScoreRankService {
    @Autowired
    UserInfoMapper userInfoMapper;
    @Autowired
    SynthesisPlanClientMapper synthesisPlanClientMapper;
    @Autowired
    SynthesisPlanScoreMapper synthesisPlanScoreMapper;
    @Autowired
    StuTheoryRecordMapper theoryRecordMapper;
    @Autowired
    TrainingService trainingService;
    @Autowired
    StuTrainingMapper stuTrainingMapper;
    @Autowired
    ScoreRankMapper scoreRankMapper;
    @Autowired
    TotalWeightMapper totalWeightMapper;
    @Autowired
    StuTheoryRecordMapper stuTheoryRecordMapper;
    @Autowired
    TrainingReportMapper trainingReportMapper;

    @Override
    @Scheduled(cron = "0 0 1 * * ?")
    public Boolean doRank() {
        //先删除当天所有的数据 防止脏数据
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        scoreRankExample.createCriteria().andUpdateDateEqualTo(getNowDate());
        scoreRankMapper.deleteByExample(scoreRankExample);
        //先查询userInfo表 获取所有学校ID
        List<String> schoolIdList = userInfoMapper.selectAllSchoolIds();
        for (int i = 0; i < schoolIdList.size(); i++) {
            List<ScoreRank> scoreRankList = new ArrayList<>();
            String schoolId = schoolIdList.get(i);
            //根据学校ID 获取所有学生userId
            UserInfoExample userInfoExample = new UserInfoExample();
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.YEAR, -1);
            Date oneYearAgo = calendar.getTime();
            userInfoExample.createCriteria().andSchoolIdEqualTo(schoolId).andRoleIdEqualTo(4).andLoginTimeBetween(oneYearAgo, new Date()).andLoginTimeIsNotNull();
            List<UserInfo> userList = userInfoMapper.selectByExample(userInfoExample);
            //查询学习成绩权重表
            Map<String, BigDecimal> clientWeight = trainingService.getClientWeight(schoolId);  //客户案例权重
            Map<String, StudentScoreWeight> studentScoreWeightMap = trainingService.getStudentScoreWeightMap(schoolId);  //学习成绩权重

            //查询成绩总览权重
            TotalWeight totalWeight = totalWeightMapper.selectByPrimaryKey(schoolId);
            if (totalWeight == null) {
                totalWeight = new TotalWeight();
                totalWeight.setKhan(BigDecimal.valueOf(0.4));
                totalWeight.setZhgh(BigDecimal.valueOf(0.4));
                totalWeight.setLlkh(BigDecimal.valueOf(0.2));
            }

            for (int j = 0; j < userList.size(); j++) {
                UserInfo userInfo = userList.get(j);
                String userId = userInfo.getUserid();
                ScoreRank scoreRank = new ScoreRank();
                scoreRank.setRankId(String.valueOf(UUID.randomUUID()));
                scoreRank.setUserid(userId);
                scoreRank.setName(userInfo.getName());
                scoreRank.setStudentId(userInfo.getStudentId());
                scoreRank.setClassId(userInfo.getClassId());
                scoreRank.setClassName(userInfo.getClassName());
                scoreRank.setSchoolId(userInfo.getSchoolId());

                //设置更新日期  年月日
                String nowDate = getNowDate();
                scoreRank.setUpdateDate(nowDate);

                //设置总数据
                //设置客户案例数据  根据userId 查询学生实训表  循环查询对象
                StuTrainingExample stuTrainingExample = new StuTrainingExample();
                stuTrainingExample.createCriteria().andUserIdEqualTo(userId);
                List<StuTraining> stuTrainings = stuTrainingMapper.selectByExample(stuTrainingExample);
                scoreRank.setFxcp(BigDecimal.ZERO);
                scoreRank.setCwfx(BigDecimal.ZERO);
                scoreRank.setXjgh(BigDecimal.ZERO);
                scoreRank.setJygh(BigDecimal.ZERO);
                scoreRank.setXfgh(BigDecimal.ZERO);
                scoreRank.setCygh(BigDecimal.ZERO);
                scoreRank.setTxgh(BigDecimal.ZERO);
                scoreRank.setBxgh(BigDecimal.ZERO);
                scoreRank.setTzgh(BigDecimal.ZERO);
                scoreRank.setSwch(BigDecimal.ZERO);
                scoreRank.setCcfp(BigDecimal.ZERO);
                scoreRank.setCccc(BigDecimal.ZERO);
                scoreRank.setKhalScore(BigDecimal.ZERO);
                if (stuTrainings != null) {
                    for (int k = 0; k < stuTrainings.size(); k++) {
                        StuTraining stuTraining = stuTrainings.get(k);
                        if (stuTraining.getChapterName() == null) {
                            continue;
                        }
                        Integer knowledgeStatus = stuTraining.getKnowledgeSummaryCompleteStatus() == null ? 0 : stuTraining.getKnowledgeSummaryCompleteStatus();
                        Integer resourceStatus = stuTraining.getResourceLearningCompleteStatus() == null ? 0 : stuTraining.getResourceLearningCompleteStatus();
                        Integer learningEvalCompleteStatus = stuTraining.getLearningEvalCompleteStatus() == null ? 9999 : stuTraining.getLearningEvalCompleteStatus();
                        Integer expStatus = stuTraining.getExpTrainingCompleteStatus() == null ? 9999 : stuTraining.getExpTrainingCompleteStatus();
                        if(knowledgeStatus>5){
                            knowledgeStatus=5;
                        }
                        if(resourceStatus>5){
                            resourceStatus=5;
                        }
                        StudentScoreWeight studentScoreWeight = studentScoreWeightMap.get(stuTraining.getChapterName());
                        BigDecimal summaryScore = studentScoreWeight.getSummaryOfKnowledgeScore().multiply(BigDecimal.valueOf(knowledgeStatus)).multiply(studentScoreWeight.getSummaryOfKnowledgeWeight());
                        BigDecimal resourceScore = studentScoreWeight.getResourceLearningScore().multiply(BigDecimal.valueOf(resourceStatus)).multiply(studentScoreWeight.getResourceLearningWeight());
                        BigDecimal learningScore= BigDecimal.valueOf(0);
                        BigDecimal experimentalScore= BigDecimal.valueOf(0);
                        if(learningEvalCompleteStatus!=9999){
                            if(learningEvalCompleteStatus<=10){
                                learningScore = BigDecimal.valueOf(100).subtract(studentScoreWeight.getLearningAssessmentScore().multiply(BigDecimal.valueOf(learningEvalCompleteStatus))).multiply(studentScoreWeight.getLearningAssessmentWeight());
                            }
                        }
                        if(expStatus!=9999){
                            if(expStatus<=5){
                                experimentalScore = BigDecimal.valueOf(100).subtract(studentScoreWeight.getExperimentalTrainingScore().multiply(BigDecimal.valueOf(expStatus))).multiply(studentScoreWeight.getExperimentalTrainingWeight());
                            }
                            //财产分配特殊处理
                            if("11".equals(stuTraining.getChapterId())){
                                if(stuTraining.getExpTrainingCompleteStatus().compareTo(100)>0){
                                    stuTraining.setExpTrainingCompleteStatus(100);
                                }
                                experimentalScore= BigDecimal.valueOf(stuTraining.getExpTrainingCompleteStatus()).multiply(studentScoreWeight.getExperimentalTrainingWeight());
                            }
                        }
                        //计算报告分数
                        String chapterId = stuTraining.getChapterId();
                        TrainingReportExample trainingReportExample = new TrainingReportExample();
                        trainingReportExample.createCriteria().andChapterIdEqualTo(chapterId).andUserIdEqualTo(userId);
                        List<TrainingReport> trainingReports = trainingReportMapper.selectByExample(trainingReportExample);
                        BigDecimal reportScore=BigDecimal.ZERO;
                        if (!trainingReports.isEmpty()) {
                            if (trainingReports.get(0).getTeacherScore() != null) {
                                reportScore = trainingReports.get(0).getTeacherScore().multiply(studentScoreWeight.getReportWeight());
                            }
                        }
                        BigDecimal sum = summaryScore.add(resourceScore).add(learningScore).add(experimentalScore).add(reportScore);
                        if ("风险测评".equals(stuTraining.getChapterName())) {
                            scoreRank.setFxcp(sum);
                        } else if ("财务分析".equals(stuTraining.getChapterName())) {
                            scoreRank.setCwfx(sum);
                        } else if ("现金规划".equals(stuTraining.getChapterName())) {
                            scoreRank.setXjgh(sum);
                        } else if ("教育规划".equals(stuTraining.getChapterName())) {
                            scoreRank.setJygh(sum);
                        } else if ("消费规划".equals(stuTraining.getChapterName())) {
                            scoreRank.setXfgh(sum);
                        } else if ("创业规划".equals(stuTraining.getChapterName())) {
                            scoreRank.setCygh(sum);
                        } else if ("退休规划".equals(stuTraining.getChapterName())) {
                            scoreRank.setTxgh(sum);
                        } else if ("保险规划".equals(stuTraining.getChapterName())) {
                            scoreRank.setBxgh(sum);
                        } else if ("投资规划".equals(stuTraining.getChapterName())) {
                            scoreRank.setTzgh(sum);
                        } else if ("税务筹划".equals(stuTraining.getChapterName())) {
                            scoreRank.setSwch(sum);
                        } else if ("财产分配".equals(stuTraining.getChapterName())) {
                            scoreRank.setCcfp(sum);
                        } else if ("财产传承".equals(stuTraining.getChapterName())) {
                            scoreRank.setCccc(sum);
                        }
                    }
                    scoreRank.setKhalScore((scoreRank.getFxcp().multiply(clientWeight.get("风险测评")))
                            .add(scoreRank.getCwfx().multiply(clientWeight.get("财务分析")))
                            .add(scoreRank.getXjgh().multiply(clientWeight.get("现金规划")))
                            .add(scoreRank.getJygh().multiply(clientWeight.get("教育规划")))
                            .add(scoreRank.getXfgh().multiply(clientWeight.get("消费规划")))
                            .add(scoreRank.getCygh().multiply(clientWeight.get("创业规划")))
                            .add(scoreRank.getTxgh().multiply(clientWeight.get("退休规划")))
                            .add(scoreRank.getBxgh().multiply(clientWeight.get("保险规划")))
                            .add(scoreRank.getTzgh().multiply(clientWeight.get("投资规划")))
                            .add(scoreRank.getSwch().multiply(clientWeight.get("税务筹划")))
                            .add(scoreRank.getCcfp().multiply(clientWeight.get("财产分配")))
                            .add(scoreRank.getCccc().multiply(clientWeight.get("财产传承"))));
                }

                //设置综合规划数据  先根据userId到综合规划客户表中获取提交状态为未审核、已审核的案例个数  再到综合规划成绩表中根据userId获取得分之和
                SynthesisPlanClientExample synthesisPlanClientExample = new SynthesisPlanClientExample();
                List<String> submitStatusList = new ArrayList<>();
                submitStatusList.add("未审核");
                submitStatusList.add("已审核");
                synthesisPlanClientExample.createCriteria().andUseridEqualTo(userId).andSubmitStatusIn(submitStatusList);
                long countCase = synthesisPlanClientMapper.countByExample(synthesisPlanClientExample);
                scoreRank.setCountCase((int) countCase);

                StuSynthesisPlanScoreAndAvgScoreDto dto = trainingService.selectStuSynthesisPlanScoreList(userId,schoolId);
                if(dto==null){
                    scoreRank.setZhghScore(BigDecimal.ZERO);
                }else {
                    scoreRank.setZhghScore(dto.getAvgScore());
                }

                //设置理论考核平均分成绩
                StuTheoryRecord stuTheoryRecord = theoryRecordMapper.selectByPrimaryKey(userId);
                if (stuTheoryRecord == null) {
                    scoreRank.setLlkhScore(BigDecimal.ZERO);
                } else {
                    scoreRank.setLlkhScore(stuTheoryRecord.getAverageScore());
                }

                //设置totalscore
                BigDecimal weightKhan = totalWeight.getKhan();
                BigDecimal weightZhgh = totalWeight.getZhgh();
                BigDecimal weightLlkh = totalWeight.getLlkh();
                if (scoreRank.getKhalScore() == null) {
                    scoreRank.setKhalScore(BigDecimal.ZERO);
                }
                if (scoreRank.getZhghScore() == null) {
                    scoreRank.setZhghScore(BigDecimal.ZERO);
                }
                if (scoreRank.getLlkhScore() == null) {
                    scoreRank.setLlkhScore(BigDecimal.ZERO);
                }
                scoreRank.setTotalScore(scoreRank.getKhalScore().multiply(weightKhan).add(scoreRank.getZhghScore().multiply(weightZhgh)).add(scoreRank.getLlkhScore().multiply(weightLlkh)));
                scoreRankList.add(scoreRank);
            }
            //总排名排序
            List<ScoreRank> totalSortedList = scoreRankList.stream()
                    .sorted((o1, o2) -> o2.getTotalScore().compareTo(o1.getTotalScore()))
                    .collect(Collectors.toList());

            for (int j = 0; j < totalSortedList.size(); j++) {
                totalSortedList.get(j).setTotalRank(j + 1);
            }

            //客户案例排序
            List<ScoreRank> khanSortedList = totalSortedList.stream()
                    .sorted((o1, o2) -> o2.getKhalScore().compareTo(o1.getKhalScore()))
                    .collect(Collectors.toList());

            for (int j = 0; j < khanSortedList.size(); j++) {
                khanSortedList.get(j).setKhalRank(j + 1);
            }

            //综合规划排序
            List<ScoreRank> zhghSortedList = khanSortedList.stream()
                    .sorted((o1, o2) -> o2.getZhghScore().compareTo(o1.getZhghScore()))
                    .collect(Collectors.toList());

            for (int j = 0; j < zhghSortedList.size(); j++) {
                zhghSortedList.get(j).setZhghRank(j + 1);
            }

            //理论考核排序
            List<ScoreRank> llkhSortedList = zhghSortedList.stream()
                    .sorted((o1, o2) -> o2.getLlkhScore().compareTo(o1.getLlkhScore()))
                    .collect(Collectors.toList());

            for (int j = 0; j < llkhSortedList.size(); j++) {
                llkhSortedList.get(j).setLlkhRank(j + 1);
            }

            //保存到数据库
            for (int j = 0; j < llkhSortedList.size(); j++) {
                scoreRankMapper.insert(llkhSortedList.get(j));
            }
        }
        return true;
    }

    @Override
    public List<ScoreRank> selectExportScoreRankList(String schoolId) {
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        ScoreRankExample.Criteria criteria = scoreRankExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId).andUpdateDateEqualTo(getNowDate());
        scoreRankExample.setOrderByClause("khal_score+zhgh_score+llkh_score desc");
        List<ScoreRank> scoreRankList = scoreRankMapper.selectByExample(scoreRankExample);
        //查询成绩总览权重
        TotalWeight totalWeight = totalWeightMapper.selectByPrimaryKey(schoolId);
        if (totalWeight == null) {
            totalWeight = new TotalWeight();
            totalWeight.setKhan(BigDecimal.valueOf(0.4));
            totalWeight.setZhgh(BigDecimal.valueOf(0.4));
            totalWeight.setLlkh(BigDecimal.valueOf(0.2));
        }
        //设置总成绩
        for (int i = 0; i < scoreRankList.size(); i++) {
            ScoreRank scoreRank = scoreRankList.get(i);
            scoreRank.setTotalScore((totalWeight.getKhan().multiply(scoreRank.getKhalScore()))
                    .add(totalWeight.getZhgh().multiply(scoreRank.getZhghScore()))
                    .add(totalWeight.getLlkh().multiply(scoreRank.getLlkhScore())));
            scoreRank.setRankId(String.valueOf(i + 1));
        }
        return scoreRankList;
    }

    @Override
    public List<StuTheoryRecord> selectExportTheoryScoreRankList(String schoolId) {
        StuTheoryRecordExample theoryRecordExample = new StuTheoryRecordExample();
        StuTheoryRecordExample.Criteria criteria = theoryRecordExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId);
        theoryRecordExample.setOrderByClause("average_score desc");
        List<StuTheoryRecord> stuTheoryRecords = stuTheoryRecordMapper.selectByExample(theoryRecordExample);
        if (!stuTheoryRecords.isEmpty()) {
            for (int i = 0; i < stuTheoryRecords.size(); i++) {
                //将排名设置到classId中 用于返回
                stuTheoryRecords.get(i).setClassId(String.valueOf(i + 1));
            }
        }

        return stuTheoryRecords;
    }


    //获取当天时间
    @Override
    public String getNowDate() {
        LocalDateTime now = LocalDateTime.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String nowDate = now.format(formatter);
        return nowDate;
    }

    @Override
    public Boolean doRankOne(String schoolId) {
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        scoreRankExample.createCriteria().andUpdateDateEqualTo(getNowDate()).andSchoolIdEqualTo(schoolId);
        scoreRankMapper.deleteByExample(scoreRankExample);
        //先查询userInfo表 获取所有学校ID
        List<ScoreRank> scoreRankList = new ArrayList<>();
        //根据学校ID 获取所有学生userId
        UserInfoExample userInfoExample = new UserInfoExample();
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.YEAR, -1);
        Date oneYearAgo = calendar.getTime();
        userInfoExample.createCriteria().andSchoolIdEqualTo(schoolId).andRoleIdEqualTo(4).andLoginTimeBetween(oneYearAgo, new Date()).andLoginTimeIsNotNull();
        List<UserInfo> userList = userInfoMapper.selectByExample(userInfoExample);
        //查询学习成绩权重表
        Map<String, BigDecimal> clientWeight = trainingService.getClientWeight(schoolId);  //客户案例权重
        Map<String, StudentScoreWeight> studentScoreWeightMap = trainingService.getStudentScoreWeightMap(schoolId);  //学习成绩权重

        //查询成绩总览权重
        TotalWeight totalWeight = totalWeightMapper.selectByPrimaryKey(schoolId);
        if (totalWeight == null) {
            totalWeight = new TotalWeight();
            totalWeight.setKhan(BigDecimal.valueOf(0.4));
            totalWeight.setZhgh(BigDecimal.valueOf(0.4));
            totalWeight.setLlkh(BigDecimal.valueOf(0.2));
        }

        for (int j = 0; j < userList.size(); j++) {
            UserInfo userInfo = userList.get(j);
            String userId = userInfo.getUserid();
            ScoreRank scoreRank = new ScoreRank();
            scoreRank.setRankId(String.valueOf(UUID.randomUUID()));
            scoreRank.setUserid(userId);
            scoreRank.setName(userInfo.getName());
            scoreRank.setStudentId(userInfo.getStudentId());
            scoreRank.setClassId(userInfo.getClassId());
            scoreRank.setClassName(userInfo.getClassName());
            scoreRank.setSchoolId(userInfo.getSchoolId());

            //设置更新日期  年月日
            String nowDate = getNowDate();
            scoreRank.setUpdateDate(nowDate);

            //设置总数据
            //设置客户案例数据  根据userId 查询学生实训表  循环查询对象
            StuTrainingExample stuTrainingExample = new StuTrainingExample();
            stuTrainingExample.createCriteria().andUserIdEqualTo(userId);
            List<StuTraining> stuTrainings = stuTrainingMapper.selectByExample(stuTrainingExample);
            scoreRank.setFxcp(BigDecimal.ZERO);
            scoreRank.setCwfx(BigDecimal.ZERO);
            scoreRank.setXjgh(BigDecimal.ZERO);
            scoreRank.setJygh(BigDecimal.ZERO);
            scoreRank.setXfgh(BigDecimal.ZERO);
            scoreRank.setCygh(BigDecimal.ZERO);
            scoreRank.setTxgh(BigDecimal.ZERO);
            scoreRank.setBxgh(BigDecimal.ZERO);
            scoreRank.setTzgh(BigDecimal.ZERO);
            scoreRank.setSwch(BigDecimal.ZERO);
            scoreRank.setCcfp(BigDecimal.ZERO);
            scoreRank.setCccc(BigDecimal.ZERO);
            scoreRank.setKhalScore(BigDecimal.ZERO);
            if (stuTrainings != null) {
                for (int k = 0; k < stuTrainings.size(); k++) {
                    StuTraining stuTraining = stuTrainings.get(k);
                    if (stuTraining.getChapterName() == null) {
                        continue;
                    }
                    Integer knowledgeStatus = stuTraining.getKnowledgeSummaryCompleteStatus() == null ? 0 : stuTraining.getKnowledgeSummaryCompleteStatus();
                    Integer resourceStatus = stuTraining.getResourceLearningCompleteStatus() == null ? 0 : stuTraining.getResourceLearningCompleteStatus();
                    Integer learningEvalCompleteStatus = stuTraining.getLearningEvalCompleteStatus() == null ? 9999 : stuTraining.getLearningEvalCompleteStatus();
                    Integer expStatus = stuTraining.getExpTrainingCompleteStatus() == null ? 9999 : stuTraining.getExpTrainingCompleteStatus();
                    if(knowledgeStatus>5){
                        knowledgeStatus=5;
                    }
                    if(resourceStatus>5){
                        resourceStatus=5;
                    }
                    StudentScoreWeight studentScoreWeight = studentScoreWeightMap.get(stuTraining.getChapterName());
                    BigDecimal summaryScore = studentScoreWeight.getSummaryOfKnowledgeScore().multiply(BigDecimal.valueOf(knowledgeStatus)).multiply(studentScoreWeight.getSummaryOfKnowledgeWeight());
                    BigDecimal resourceScore = studentScoreWeight.getResourceLearningScore().multiply(BigDecimal.valueOf(resourceStatus)).multiply(studentScoreWeight.getResourceLearningWeight());
                    BigDecimal learningScore= BigDecimal.valueOf(0);
                    BigDecimal experimentalScore= BigDecimal.valueOf(0);
                    if(learningEvalCompleteStatus!=9999){
                        if(learningEvalCompleteStatus<=10){
                            learningScore = BigDecimal.valueOf(100).subtract(studentScoreWeight.getLearningAssessmentScore().multiply(BigDecimal.valueOf(learningEvalCompleteStatus))).multiply(studentScoreWeight.getLearningAssessmentWeight());
                        }
                    }
                    if(expStatus!=9999){
                        if(expStatus<=5){
                            experimentalScore = BigDecimal.valueOf(100).subtract(studentScoreWeight.getExperimentalTrainingScore().multiply(BigDecimal.valueOf(expStatus))).multiply(studentScoreWeight.getExperimentalTrainingWeight());
                        }
                        //财产分配特殊处理
                        if("11".equals(stuTraining.getChapterId())){
                            if(stuTraining.getExpTrainingCompleteStatus().compareTo(100)>0){
                                stuTraining.setExpTrainingCompleteStatus(100);
                            }
                            experimentalScore= BigDecimal.valueOf(stuTraining.getExpTrainingCompleteStatus()).multiply(studentScoreWeight.getExperimentalTrainingWeight());
                        }
                    }
                    //计算报告分数
                    String chapterId = stuTraining.getChapterId();
                    TrainingReportExample trainingReportExample = new TrainingReportExample();
                    trainingReportExample.createCriteria().andChapterIdEqualTo(chapterId).andUserIdEqualTo(userId);
                    List<TrainingReport> trainingReports = trainingReportMapper.selectByExample(trainingReportExample);
                    BigDecimal reportScore=BigDecimal.ZERO;
                    if (!trainingReports.isEmpty()) {
                        if (trainingReports.get(0).getTeacherScore() != null) {  //todo 报告得分为0导致更新成绩报错，这里加了一个判断
                            reportScore = trainingReports.get(0).getTeacherScore().multiply(studentScoreWeight.getReportWeight());
                        }
                    }

                    BigDecimal sum = summaryScore.add(resourceScore).add(learningScore).add(experimentalScore).add(reportScore);
                    if ("风险测评".equals(stuTraining.getChapterName())) {
                        scoreRank.setFxcp(sum);
                    } else if ("财务分析".equals(stuTraining.getChapterName())) {
                        scoreRank.setCwfx(sum);
                    } else if ("现金规划".equals(stuTraining.getChapterName())) {
                        scoreRank.setXjgh(sum);
                    } else if ("教育规划".equals(stuTraining.getChapterName())) {
                        scoreRank.setJygh(sum);
                    } else if ("消费规划".equals(stuTraining.getChapterName())) {
                        scoreRank.setXfgh(sum);
                    } else if ("创业规划".equals(stuTraining.getChapterName())) {
                        scoreRank.setCygh(sum);
                    } else if ("退休规划".equals(stuTraining.getChapterName())) {
                        scoreRank.setTxgh(sum);
                    } else if ("保险规划".equals(stuTraining.getChapterName())) {
                        scoreRank.setBxgh(sum);
                    } else if ("投资规划".equals(stuTraining.getChapterName())) {
                        scoreRank.setTzgh(sum);
                    } else if ("税务筹划".equals(stuTraining.getChapterName())) {
                        scoreRank.setSwch(sum);
                    } else if ("财产分配".equals(stuTraining.getChapterName())) {
                        scoreRank.setCcfp(sum);
                    } else if ("财产传承".equals(stuTraining.getChapterName())) {
                        scoreRank.setCccc(sum);
                    }
                }
                scoreRank.setKhalScore((scoreRank.getFxcp().multiply(clientWeight.get("风险测评")))
                        .add(scoreRank.getCwfx().multiply(clientWeight.get("财务分析")))
                        .add(scoreRank.getXjgh().multiply(clientWeight.get("现金规划")))
                        .add(scoreRank.getJygh().multiply(clientWeight.get("教育规划")))
                        .add(scoreRank.getXfgh().multiply(clientWeight.get("消费规划")))
                        .add(scoreRank.getCygh().multiply(clientWeight.get("创业规划")))
                        .add(scoreRank.getTxgh().multiply(clientWeight.get("退休规划")))
                        .add(scoreRank.getBxgh().multiply(clientWeight.get("保险规划")))
                        .add(scoreRank.getTzgh().multiply(clientWeight.get("投资规划")))
                        .add(scoreRank.getSwch().multiply(clientWeight.get("税务筹划")))
                        .add(scoreRank.getCcfp().multiply(clientWeight.get("财产分配")))
                        .add(scoreRank.getCccc().multiply(clientWeight.get("财产传承"))));
            }

            //设置综合规划数据  先根据userId到综合规划客户表中获取提交状态为未审核、已审核的案例个数  再到综合规划成绩表中根据userId获取得分之和
            SynthesisPlanClientExample synthesisPlanClientExample = new SynthesisPlanClientExample();
            List<String> submitStatusList = new ArrayList<>();
            submitStatusList.add("未审核");
            submitStatusList.add("已审核");
            synthesisPlanClientExample.createCriteria().andUseridEqualTo(userId).andSubmitStatusIn(submitStatusList);
            long countCase = synthesisPlanClientMapper.countByExample(synthesisPlanClientExample);
            scoreRank.setCountCase((int) countCase);

            StuSynthesisPlanScoreAndAvgScoreDto dto = trainingService.selectStuSynthesisPlanScoreList(userId,schoolId);
            if(dto==null){
                scoreRank.setZhghScore(BigDecimal.ZERO);
            }else {
                scoreRank.setZhghScore(dto.getAvgScore());
            }

            //设置理论考核平均分成绩
            StuTheoryRecord stuTheoryRecord = theoryRecordMapper.selectByPrimaryKey(userId);
            if (stuTheoryRecord == null) {
                scoreRank.setLlkhScore(BigDecimal.ZERO);
            } else {
                scoreRank.setLlkhScore(stuTheoryRecord.getAverageScore());
            }

            //设置totalscore
            BigDecimal weightKhan = totalWeight.getKhan();
            BigDecimal weightZhgh = totalWeight.getZhgh();
            BigDecimal weightLlkh = totalWeight.getLlkh();
            if (scoreRank.getKhalScore() == null) {
                scoreRank.setKhalScore(BigDecimal.ZERO);
            }
            if (scoreRank.getZhghScore() == null) {
                scoreRank.setZhghScore(BigDecimal.ZERO);
            }
            if (scoreRank.getLlkhScore() == null) {
                scoreRank.setLlkhScore(BigDecimal.ZERO);
            }
            scoreRank.setTotalScore(scoreRank.getKhalScore().multiply(weightKhan).add(scoreRank.getZhghScore().multiply(weightZhgh)).add(scoreRank.getLlkhScore().multiply(weightLlkh)));
            scoreRankList.add(scoreRank);
        }
        //总排名排序
        List<ScoreRank> totalSortedList = scoreRankList.stream()
                .sorted((o1, o2) -> o2.getTotalScore().compareTo(o1.getTotalScore()))
                .collect(Collectors.toList());

        for (int j = 0; j < totalSortedList.size(); j++) {
            totalSortedList.get(j).setTotalRank(j + 1);
        }

        //客户案例排序
        List<ScoreRank> khanSortedList = totalSortedList.stream()
                .sorted((o1, o2) -> o2.getKhalScore().compareTo(o1.getKhalScore()))
                .collect(Collectors.toList());

        for (int j = 0; j < khanSortedList.size(); j++) {
            khanSortedList.get(j).setKhalRank(j + 1);
        }

        //综合规划排序
        List<ScoreRank> zhghSortedList = khanSortedList.stream()
                .sorted((o1, o2) -> o2.getZhghScore().compareTo(o1.getZhghScore()))
                .collect(Collectors.toList());

        for (int j = 0; j < zhghSortedList.size(); j++) {
            zhghSortedList.get(j).setZhghRank(j + 1);
        }

        //理论考核排序
        List<ScoreRank> llkhSortedList = zhghSortedList.stream()
                .sorted((o1, o2) -> o2.getLlkhScore().compareTo(o1.getLlkhScore()))
                .collect(Collectors.toList());

        for (int j = 0; j < llkhSortedList.size(); j++) {
            llkhSortedList.get(j).setLlkhRank(j + 1);
        }

        //保存到数据库
        for (int j = 0; j < llkhSortedList.size(); j++) {
            scoreRankMapper.insert(llkhSortedList.get(j));
        }
        return true;
    }


}
