package com.sztzjy.money_management.service.impl;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.money_management.entity.*;
import com.sztzjy.money_management.entity.dto.CalcAuthScore;
import com.sztzjy.money_management.entity.dto.StuAuthTestDto;
import com.sztzjy.money_management.entity.dto.StuTheoryExamDetailDto;
import com.sztzjy.money_management.entity.dto.StuTheoryTestDto;
import com.sztzjy.money_management.mapper.*;
import com.sztzjy.money_management.service.TheoryTestService;
import com.sztzjy.money_management.util.PageUtil;
import com.sztzjy.money_management.util.ResultEntity;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * @Author xcj
 * @Date 2024/8/22
 */
@Service
public class TheoryTestServiceImpl implements TheoryTestService {
    @Autowired
    private StuTheoryExamMapper theoryExamMapper;
    @Autowired
    private StuTheoryRecordMapper theoryRecordMapper;
    @Autowired
    private UserInfoMapper userMapper;
    @Autowired
    private ObjectiveQuestionMapper objectiveQuestionMapper;
    @Autowired
    private StuErrorMapper errorMapper;


    //交卷后 更新理论考试记录数据
    @Override
    public int endTheoryTest(StuTheoryTestDto theoryTestDto) {
        StuTheoryExamExample example = new StuTheoryExamExample();
        example.createCriteria().andUserIdEqualTo(theoryTestDto.getUserId()).andSourceEqualTo(theoryTestDto.getSource());
        example.setOrderByClause("exam_time DESC");
        List<StuTheoryExamWithBLOBs> stuTheoryExams = theoryExamMapper.selectByExampleWithBLOBs(example);
        if (stuTheoryExams == null || stuTheoryExams.isEmpty()) {
            return 0;
        }
        StuTheoryExamWithBLOBs stuTheoryExam = stuTheoryExams.get(0);
        stuTheoryExam.setExamDuration(theoryTestDto.getExamDuration());
        stuTheoryExam.setScore(theoryTestDto.getScore());
        stuTheoryExam.setAccuracy(theoryTestDto.getAccuracy());
        ArrayList<String> list = new ArrayList<>();
        list.addAll(theoryTestDto.getSingleErrorIds());
        list.addAll(theoryTestDto.getMultipleErrorIds());
        list.addAll(theoryTestDto.getJudgeErrorIds());

        String singleStuAnswer = theoryTestDto.getSingleStuAnswer();
        stuTheoryExam.setSingleStuAnswer(singleStuAnswer);

        String multipleStuAnswer = theoryTestDto.getMultipleStuAnswer();
        stuTheoryExam.setMultipleStuAnswer(multipleStuAnswer);

        String judgeStuAnswer = theoryTestDto.getJudgeStuAnswer();
        stuTheoryExam.setJudgeStuAnswer(judgeStuAnswer);

        String stingList = String.valueOf(list.toString()).replace("[", "").replace("]", "");
        stuTheoryExam.setErrorIds(stingList);
        stuTheoryExam.setAnswered(true);
        stuTheoryExam.setSource(theoryTestDto.getSource());
        int i = theoryExamMapper.updateByPrimaryKeyWithBLOBs(stuTheoryExam);

        if (theoryTestDto.getSource().equals("理论考试")) {
            //更新错题集
            StuErrorExample errorExample = new StuErrorExample();
            errorExample.createCriteria().andUserIdEqualTo(theoryTestDto.getUserId());
            List<StuErrorWithBLOBs> stuErrors = errorMapper.selectByExampleWithBLOBs(errorExample);
            if (stuErrors == null || stuErrors.isEmpty()) {
                StuErrorWithBLOBs stuError = new StuErrorWithBLOBs();
                stuError.setErrorId(IdUtil.randomUUID());
                stuError.setUserId(theoryTestDto.getUserId());

                String singleError = String.valueOf(theoryTestDto.getSingleErrorIds()).replace("[", "").replace("]", "");
                stuError.setSingleIds(singleError);

                String multipleError = String.valueOf(theoryTestDto.getMultipleErrorIds()).replace("[", "").replace("]", "");
                stuError.setMultipleIds(multipleError);

                String judgeError = String.valueOf(theoryTestDto.getJudgeErrorIds()).replace("[", "").replace("]", "");
                stuError.setJudgeIds(judgeError);
                errorMapper.insert(stuError);
            } else {
                StuErrorWithBLOBs stuError = stuErrors.get(0);
                if (StringUtils.isNotBlank(stuError.getSingleIds())) {
                    List<String> stringList = Arrays.asList(stuError.getSingleIds().substring(1, stuError.getSingleIds().length() - 1).split(", "));
                    List<String> singleErrorIds = theoryTestDto.getSingleErrorIds();

                    List<String> combinedList = new ArrayList<>(stringList);
                    combinedList.addAll(singleErrorIds);
                    Set<String> uniqueSet = new HashSet<>(combinedList);
                    List<String> uniqueList = new ArrayList<>(uniqueSet);
                    String stringUnique = String.valueOf(uniqueList).replace("[", "").replace("]", "");
                    stuError.setSingleIds(stringUnique);
                } else {
                    String stringError = String.valueOf(theoryTestDto.getSingleErrorIds()).replace("[", "").replace("]", "");
                    stuError.setSingleIds(stringError);
                }
                if (StringUtils.isNotBlank(stuError.getMultipleIds())) {
                    List<String> stringList = Arrays.asList(stuError.getMultipleIds().substring(1, stuError.getMultipleIds().length() - 1).split(", "));
                    List<String> multipleErrorIds = theoryTestDto.getMultipleErrorIds();

                    List<String> combinedList = new ArrayList<>(stringList);
                    combinedList.addAll(multipleErrorIds);
                    Set<String> uniqueSet = new HashSet<>(combinedList);
                    List<String> uniqueList = new ArrayList<>(uniqueSet);
                    String stringUnique = String.valueOf(uniqueList).replace("[", "").replace("]", "");
                    stuError.setMultipleIds(stringUnique);
                } else {
                    String stringMultipleError = String.valueOf(theoryTestDto.getMultipleErrorIds()).replace("[", "").replace("]", "");
                    stuError.setMultipleIds(stringMultipleError);
                }
                if (StringUtils.isNotBlank(stuError.getJudgeIds())) {
                    List<String> stringList = Arrays.asList(stuError.getJudgeIds().substring(1, stuError.getJudgeIds().length() - 1).split(", "));
                    List<String> judgeErrorIds = theoryTestDto.getJudgeErrorIds();
                    List<String> combinedList = new ArrayList<>(stringList);
                    combinedList.addAll(judgeErrorIds);
                    Set<String> uniqueSet = new HashSet<>(combinedList);
                    List<String> uniqueList = new ArrayList<>(uniqueSet);
                    String stringUnique = String.valueOf(uniqueList).replace("[", "").replace("]", "");
                    stuError.setJudgeIds(stringUnique);
                } else {
                    String stringJudgeError = String.valueOf(theoryTestDto.getJudgeErrorIds()).replace("[", "").replace("]", "");
                    stuError.setJudgeIds(stringJudgeError);
                }
                errorMapper.updateByPrimaryKeySelective(stuError);
            }
            UpdateTheoryRecord(theoryTestDto.getUserId(), theoryTestDto.getScore(), theoryTestDto.getExamDuration());
        }
        return i;
    }

    public void UpdateTheoryRecord(String userId, BigDecimal score, Integer examDuration) {
        StuTheoryRecord record = theoryRecordMapper.selectByPrimaryKey(userId);
        if (record != null) {
            record.setExamCount(record.getExamCount() + 1);  //原有的+这次
            if (record.getHighestScore() == null) {
                record.setHighestScore(score);
            } else {
                if (record.getHighestScore().compareTo(score) < 0) {
                    record.setHighestScore(score);
                }
            }
            if (record.getLowestScore() == null) {
                record.setLowestScore(score);
            } else {
                if (record.getLowestScore().compareTo(score) > 0) {
                    record.setLowestScore(score);
                }
            }
            if (record.getTotalDuration() == null) {
                record.setTotalDuration(BigDecimal.valueOf(examDuration));
            } else {
                BigDecimal totalDuration = record.getTotalDuration();
                record.setTotalDuration(BigDecimal.valueOf(examDuration + totalDuration.intValue()));
            }
            if (record.getAverageScore() == null) {
                record.setAverageScore(score);
            } else {
                StuTheoryExamExample example = new StuTheoryExamExample();
                example.createCriteria().andUserIdEqualTo(userId).andAnsweredEqualTo(true);
                List<StuTheoryExamWithBLOBs> stuTheoryExamWithBLOBs = theoryExamMapper.selectByExampleWithBLOBs(example);
                BigDecimal totalScore = stuTheoryExamWithBLOBs.stream()
                        .map(StuTheoryExamWithBLOBs::getScore)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                BigDecimal avgScore = totalScore.divide(BigDecimal.valueOf(stuTheoryExamWithBLOBs.size()), 2, RoundingMode.HALF_UP);
                record.setAverageScore(avgScore);
            }
        }
        theoryRecordMapper.updateByPrimaryKey(record);
    }

    @Override
    public Date getStartTime(String userId, String source) {
        StuTheoryExamExample example = new StuTheoryExamExample();
        example.createCriteria().andUserIdEqualTo(userId).andSourceEqualTo(source);
        example.setOrderByClause("exam_time DESC");
        List<StuTheoryExamWithBLOBs> stuTheoryExams = theoryExamMapper.selectByExampleWithBLOBs(example);
        if (stuTheoryExams.isEmpty()) {  //之前没有进行考试
            return null;
        } else {  //根据考试时间排序  查询最近的一条
            StuTheoryExamWithBLOBs stuTheoryExam = stuTheoryExams.get(0);
            if (stuTheoryExam.getAnswered()) {
                return null;
            }
            Date currentTime = new Date();
            Date twoHoursAgo = new Date(currentTime.getTime() - (2 * 60 * 60 * 1000)); // 当前时间减去两个小时
            if (stuTheoryExam.getExamTime().before(twoHoursAgo)) {  //已超出两个小时  从新生成新试卷
                return null;
            } else {  //在两个小时内 返回最近的一条试卷
                return stuTheoryExam.getExamTime();
            }
        }
    }


    /**
     * 返回认证考试记录
     *
     * @author xcj
     * @Date 2024/8/26
     */
    @Override
    public ResultEntity<PageInfo<StuAuthTestDto>> getAuthTestRecord(Integer index, Integer size, String userId, String source) {
        StuTheoryExamExample stuTheoryExamExample = new StuTheoryExamExample();
        stuTheoryExamExample.createCriteria().andUserIdEqualTo(userId).andSourceEqualTo(source);
        List<StuTheoryExamWithBLOBs> dataList = theoryExamMapper.selectByExampleWithBLOBs(stuTheoryExamExample);

        ArrayList<StuAuthTestDto> stuAuthTestDtoList = new ArrayList<>();
        if (dataList != null && !dataList.isEmpty()) {
            for (StuTheoryExamWithBLOBs data : dataList) {
                StuAuthTestDto dto = new StuAuthTestDto();
                BeanUtils.copyProperties(data, dto);

                int singleSize = 0;
                int multipleSize = 0;
                int judgeSize = 0;
                if (StringUtils.isNotBlank(data.getSingleIds())) {
                    singleSize = data.getSingleIds().split(", ").length;
                }
                if (StringUtils.isNotBlank(data.getMultipleIds())) {
                    multipleSize = data.getMultipleIds().split(", ").length;
                }
                if (StringUtils.isNotBlank(data.getJudgeIds())) {
                    judgeSize = data.getJudgeIds().split(", ").length;
                }
                int totalSize = singleSize + multipleSize + judgeSize;

                String name = userMapper.selectByPrimaryKey(data.getUserId()).getName();
                List<CalcAuthScore> list = new ArrayList<>();
                CalcAuthScore singleScore = new CalcAuthScore();
                singleScore.setType("单选题");
                CalcAuthScore multipleScore = new CalcAuthScore();
                multipleScore.setType("多选题");
                CalcAuthScore judgeScore = new CalcAuthScore();
                judgeScore.setType("判断题");

                int singleCount = 0;
                int multipleCount = 0;
                int judgeCount = 0;
                if (!StringUtils.isBlank(data.getErrorIds())) {
                    List<String> errorList = Arrays.asList(data.getErrorIds().split(", "));
                    ObjectiveQuestionExample example = new ObjectiveQuestionExample();
                    example.createCriteria().andObjectiveIdIn(errorList);
                    List<ObjectiveQuestionWithBLOBs> dataErrors = objectiveQuestionMapper.selectByExampleWithBLOBs(example);

                    for (ObjectiveQuestionWithBLOBs dataError : dataErrors) {
                        if (dataError.getType() == 0) {
                            singleCount++;
                        } else if (dataError.getType() == 1) {
                            multipleCount++;
                        } else if (dataError.getType() == 2) {
                            judgeCount++;
                        }
                    }
                }

                switch (source) {
                    case "理论知识":
                        singleScore = calculateScore(singleSize, singleCount, BigDecimal.valueOf(0.6), totalSize);
                        multipleScore = calculateScore(multipleSize, multipleCount, BigDecimal.valueOf(0.3), totalSize);
                        judgeScore = calculateScore(judgeSize, judgeCount, BigDecimal.valueOf(0.1), totalSize);
                        break;
                    case "专业能力":
                        singleScore = calculateScore(singleSize, singleCount, BigDecimal.ONE, totalSize);
                        break;
                    default:
                        singleScore = calculateScore(singleSize, singleCount, BigDecimal.valueOf(0.9), totalSize);
                        multipleScore = calculateScore(multipleSize, multipleCount, BigDecimal.valueOf(0.4), totalSize);
                        judgeScore = calculateScore(judgeSize, judgeCount, BigDecimal.valueOf(0.15), totalSize);
                        break;
                }

                BigDecimal totalScore = singleScore.getScore()
                        .add(multipleScore.getScore())
                        .add(judgeScore.getScore());
                dto.setTotalScore(totalScore.setScale(0, RoundingMode.HALF_UP));

                list.add(singleScore);
                list.add(multipleScore);
                list.add(judgeScore);
                dto.setCalcAuthScoreList(list);
                dto.setName(name);
                stuAuthTestDtoList.add(dto);
            }
        }
        PageInfo pageInfo = PageUtil.pageHelper(stuAuthTestDtoList, index, size);
        return new ResultEntity<>(pageInfo);
    }

    private CalcAuthScore calculateScore(int questionSize, int errorCount, BigDecimal weight, int totalSize) {
        CalcAuthScore score = new CalcAuthScore();
        if (questionSize == 0) {
            score.setAccuracy("0");
            score.setScore(BigDecimal.ZERO);
        } else {
            BigDecimal accuracy = BigDecimal.valueOf(questionSize - errorCount)
                    .divide(BigDecimal.valueOf(totalSize), RoundingMode.HALF_UP)
                    .multiply(BigDecimal.valueOf(100))
                    .setScale(2, RoundingMode.HALF_UP);
            score.setAccuracy(String.valueOf(questionSize - errorCount));
            score.setScore(accuracy.multiply(weight).setScale(2, RoundingMode.HALF_UP));
        }
        return score;
    }


//    @Override   //原始写法
//    public ResultEntity<StuAuthTestDto> getAuthTestRecord(String theoryExamId) {
//        StuAuthTestDto dto = new StuAuthTestDto();
//        StuTheoryExamWithBLOBs data = theoryExamMapper.selectByPrimaryKey(theoryExamId);
//        BeanUtils.copyProperties(data, dto);
//        String source = data.getSource();
//        int singleSize = data.getSingleIds().split(",").length;
//        int multipleSize = data.getMultipleIds().split(",").length;
//        int judgeSize = data.getJudgeIds().split(",").length;
//        String name = userMapper.selectByPrimaryKey(data.getUserId()).getName();
//
//        int totalSize = singleSize + multipleSize + judgeSize;
//        List<CalcAuthScore> list = new ArrayList<>();
//        CalcAuthScore singleScore = new CalcAuthScore();
//        singleScore.setType("单选题");
//        CalcAuthScore multipleScore = new CalcAuthScore();
//        multipleScore.setType("多选题");
//        CalcAuthScore judgeScore = new CalcAuthScore();
//        judgeScore.setType("判断题");
//        if (StringUtils.isBlank(data.getErrorIds())) { //没有错题全对的情况
//            if (source.equals("理论知识")) { //60 30 10
//                singleScore.setAccuracy("60");
//                singleScore.setScore(BigDecimal.valueOf(60));
//                multipleScore.setAccuracy("30");
//                multipleScore.setScore(BigDecimal.valueOf(30));
//                judgeScore.setAccuracy("10");
//                judgeScore.setScore(BigDecimal.valueOf(10));
//            }
//            if (source.equals("专业能力")) { // 100
//                singleScore.setAccuracy("100");
//                singleScore.setScore(BigDecimal.valueOf(100));
//            } else { //90 40 15
//                singleScore.setAccuracy("90");
//                singleScore.setScore(BigDecimal.valueOf(90));
//                multipleScore.setAccuracy("40");
//                multipleScore.setScore(BigDecimal.valueOf(40));
//                judgeScore.setAccuracy("15");
//                judgeScore.setScore(BigDecimal.valueOf(15));
//                dto.setTotalScore(BigDecimal.valueOf(100));
//            }
//        } else {
//            List<String> errorList = Arrays.asList(data.getErrorIds().split(","));
//            ObjectiveQuestionExample example = new ObjectiveQuestionExample();
//            example.createCriteria().andObjectiveIdIn(errorList);
//            List<ObjectiveQuestionWithBLOBs> dataErrors = objectiveQuestionMapper.selectByExampleWithBLOBs(example);
//            int singleCount = 0;
//            int multipleCount = 0;
//            int judgeCount = 0;
//            for (ObjectiveQuestionWithBLOBs dataError : dataErrors) {
//                Integer type = dataError.getType();
//                if (type == 0) {
//                    singleCount++;
//                }
//                if (type == 1) {
//                    multipleCount++;
//                }
//                if (type == 2) {
//                    judgeCount++;
//                }
//            }
//            if (source.equals("理论知识")) { //60 30 10
//                BigDecimal bigDecimal = BigDecimal.valueOf(singleSize - singleCount)
//                        .divide(BigDecimal.valueOf(totalSize), RoundingMode.HALF_UP)
//                        .multiply(BigDecimal.valueOf(100))
//                        .setScale(2, RoundingMode.HALF_UP);
//                singleScore.setAccuracy(String.valueOf(singleSize - singleCount));
//                singleScore.setScore(bigDecimal);
//
//                BigDecimal bigDecimal2 = BigDecimal.valueOf(multipleSize - multipleCount)
//                        .divide(BigDecimal.valueOf(totalSize), RoundingMode.HALF_UP)
//                        .multiply(BigDecimal.valueOf(100))
//                        .setScale(2, RoundingMode.HALF_UP);
//                multipleScore.setAccuracy(String.valueOf(multipleSize - multipleCount));
//                multipleScore.setScore(bigDecimal2);
//
//                BigDecimal bigDecimal3 = BigDecimal.valueOf(judgeSize - judgeCount)
//                        .divide(BigDecimal.valueOf(totalSize), RoundingMode.HALF_UP)
//                        .multiply(BigDecimal.valueOf(100))
//                        .setScale(2, RoundingMode.HALF_UP);
//                judgeScore.setAccuracy(String.valueOf(judgeSize - judgeCount));
//                judgeScore.setScore(bigDecimal3);
//                BigDecimal tatol = bigDecimal.add(bigDecimal2.add(bigDecimal3))
//                        .divide(BigDecimal.valueOf(totalSize))
//                        .multiply(BigDecimal.valueOf(100))
//                        .setScale(0, RoundingMode.HALF_UP);  //保留整数 四舍五入
//                dto.setTotalScore(tatol);
//            }
//            if (source.equals("专业能力")) { // 100
//                BigDecimal bigDecimal = BigDecimal.valueOf(singleSize - singleCount)
//                        .divide(BigDecimal.valueOf(totalSize), RoundingMode.HALF_UP)
//                        .multiply(BigDecimal.valueOf(100))
//                        .setScale(2, RoundingMode.HALF_UP);
//                singleScore.setAccuracy(String.valueOf(singleSize - singleCount));
//                singleScore.setScore(bigDecimal);
//                BigDecimal tatol = bigDecimal.divide(BigDecimal.valueOf(totalSize))
//                        .multiply(BigDecimal.valueOf(100))
//                        .setScale(0, RoundingMode.HALF_UP);  //保留整数 四舍五入
//                dto.setTotalScore(tatol);
//            } else { //90 40 15
//                BigDecimal bigDecimal = BigDecimal.valueOf(singleSize - singleCount)
//                        .divide(BigDecimal.valueOf(totalSize), RoundingMode.HALF_UP)
//                        .multiply(BigDecimal.valueOf(100))
//                        .setScale(2, RoundingMode.HALF_UP);
//                singleScore.setAccuracy(String.valueOf(singleSize - singleCount));
//                singleScore.setScore(bigDecimal);
//
//                BigDecimal bigDecimal2 = BigDecimal.valueOf(multipleSize - multipleCount)
//                        .divide(BigDecimal.valueOf(totalSize), RoundingMode.HALF_UP)
//                        .multiply(BigDecimal.valueOf(100))
//                        .setScale(2, RoundingMode.HALF_UP);
//                multipleScore.setAccuracy(String.valueOf(multipleSize - multipleCount));
//                multipleScore.setScore(bigDecimal2);
//
//                BigDecimal bigDecimal3 = BigDecimal.valueOf(judgeSize - judgeCount)
//                        .divide(BigDecimal.valueOf(totalSize), RoundingMode.HALF_UP)
//                        .multiply(BigDecimal.valueOf(100))
//                        .setScale(2, RoundingMode.HALF_UP);
//                judgeScore.setAccuracy(String.valueOf(judgeSize - judgeCount));
//                judgeScore.setScore(bigDecimal3);
//                BigDecimal tatol = bigDecimal.add(bigDecimal2.add(bigDecimal3))
//                        .divide(BigDecimal.valueOf(totalSize))
//                        .multiply(BigDecimal.valueOf(100))
//                        .setScale(0, RoundingMode.HALF_UP);  //保留整数 四舍五入
//                dto.setTotalScore(tatol);
//            }
//        }
//        list.add(singleScore);
//        list.add(multipleScore);
//        list.add(judgeScore);
//        dto.setCalcAuthScoreList(list);
//        dto.setName(name);
//        return new ResultEntity<StuAuthTestDto>(dto);
//    }

    @Override
    public PageInfo<StuTheoryExamWithBLOBs> getTheoryTestList(String userId, Integer index, Integer size) {
        PageHelper.startPage(index, size);
        StuTheoryExamExample example = new StuTheoryExamExample();
        example.createCriteria().andUserIdEqualTo(userId).andSourceEqualTo("理论考试");
        example.setOrderByClause("exam_time desc");
        List<StuTheoryExamWithBLOBs> stuTheoryExams = theoryExamMapper.selectByExampleWithBLOBs(example);
        PageInfo<StuTheoryExamWithBLOBs> pageInfo = new PageInfo<>(stuTheoryExams);
        return pageInfo;
    }

    //开始考试 1 先查询考试时间最近的一堂考试 判断是否超过2小时 如果没超过 根据理论考试表中的题目ID和答案进行回显
    //                                                如果超过了两小时 定时方法算分将数据保存到理论考试表和理论记录表中 如果根据userid查询理论记录表中数据未null  则需要创建
    //                                                之前没有进行考试 从客观题库表中查询 35道单选，每题2分，5道多选每题4分，10道判断每题1分 并将id保存至理论考试表中
    //       注意 开始考试后 要生成理论记录表基础数据
    @Override
    public List<ObjectiveQuestionWithBLOBs> startTheoryTest(String userId, String schoolId, String source) throws IOException {
        StuTheoryExamExample example = new StuTheoryExamExample();
        example.createCriteria().andUserIdEqualTo(userId).andSourceEqualTo(source);
        example.setOrderByClause("exam_time DESC");
        List<StuTheoryExamWithBLOBs> stuTheoryExams = theoryExamMapper.selectByExampleWithBLOBs(example);
        if (stuTheoryExams.isEmpty()) {  //之前没有进行考试
            List<ObjectiveQuestionWithBLOBs> objectiveQuestionList = GenerateTest(userId, schoolId, source);
            //理论考试需要额外增加一张记录表
            if (source.equals("理论考试")) {
                GenerateTheoryRecordFirst(userId);
            }
            return objectiveQuestionList;
        } else {  //根据考试时间排序  查询最近的一条
            StuTheoryExamWithBLOBs stuTheoryExam = stuTheoryExams.get(0);
            Date currentTime = new Date();
            Date twoHoursAgo = new Date(currentTime.getTime() - (2 * 60 * 60 * 1000)); // 当前时间减去两个小时
            if (stuTheoryExam.getAnswered()) {
                List<ObjectiveQuestionWithBLOBs> objectiveQuestionList = GenerateTest(userId, schoolId, source);
                return objectiveQuestionList;
            }
            if (stuTheoryExam.getExamTime().before(twoHoursAgo)) {  //已超出两个小时  从新生成新试卷
                List<ObjectiveQuestionWithBLOBs> objectiveQuestionList = GenerateTest(userId, schoolId, source);
                return objectiveQuestionList;
            } else {  //在两个小时内 返回最近的一条试卷
                String singleIds = stuTheoryExam.getSingleIds();
                String multipleIds = stuTheoryExam.getMultipleIds();
                String judgeIds = stuTheoryExam.getJudgeIds();

                List<String> singleIdList = Arrays.asList(singleIds.split(", "));
                System.out.println(Collections.singletonList(singleIdList).size());

                List<ObjectiveQuestionWithBLOBs> returnQuestionList = new ArrayList<>();
                if (StringUtils.isNotBlank(multipleIds)) {
                    List<String> multipleIdList = Arrays.asList(multipleIds.split(", "));
                    List<ObjectiveQuestionWithBLOBs> mulQuestionList = selectBatchByIdList(multipleIdList);
                    returnQuestionList.addAll(mulQuestionList);
                }
                if (StringUtils.isNotBlank(judgeIds)) {
                    List<String> judgeIdList = Arrays.asList(judgeIds.split(", "));
                    List<ObjectiveQuestionWithBLOBs> judgeQuestionList = selectBatchByIdList(judgeIdList);
                    returnQuestionList.addAll(judgeQuestionList);
                }
                List<ObjectiveQuestionWithBLOBs> singleQuestionList = selectBatchByIdList(singleIdList);
                returnQuestionList.addAll(singleQuestionList);

                // 按照题型排序，0:单选，1:多选，2:判断
                returnQuestionList.sort(Comparator.comparingInt(ObjectiveQuestion::getType));

                // 设置source字段从1开始
                for (int i = 0; i < returnQuestionList.size(); i++) {
                    int i1 = i + 1;
                    returnQuestionList.get(i).setSource(String.valueOf(i1));  // source从1开始
                }
                return returnQuestionList;
            }
        }
    }

    @Override
    public List<StuTheoryExamDetailDto> getTheoryTestDetail(String theoryExamId, Integer index, Integer size) throws IOException {
        ArrayList<String> idsList = new ArrayList<>();

        StuTheoryExamWithBLOBs theoryExam = theoryExamMapper.selectByPrimaryKey(theoryExamId);

        String singleIds = theoryExam.getSingleIds();
        if (StringUtils.isNotBlank(singleIds)) {
//            List<String> singleIdList = Arrays.asList(singleIds.substring(1, singleIds.length() - 1).split(", "));
            List<String> singleIdList = Arrays.asList(singleIds.split(", "));
            idsList.addAll(singleIdList);
        }

        String multipleIds = theoryExam.getMultipleIds();
        if (StringUtils.isNotBlank(multipleIds)) {
//            List<String> multipleIdList = Arrays.asList(multipleIds.substring(1, multipleIds.length() - 1).split(", "));
            List<String> multipleIdList = Arrays.asList(multipleIds.split(", "));
            idsList.addAll(multipleIdList);
        }

        String judgeIds = theoryExam.getJudgeIds();
        if (StringUtils.isNotBlank(judgeIds)) {
//            List<String> judgeIdList = Arrays.asList(judgeIds.substring(1, judgeIds.length() - 1).split(", "));
            List<String> judgeIdList = Arrays.asList(judgeIds.split(", "));
            idsList.addAll(judgeIdList);
        }

        ArrayList<String> answerList = new ArrayList<>();
        if (StringUtils.isNotBlank(theoryExam.getSingleStuAnswer())) {
//            String[] singleSplit = theoryExam.getSingleStuAnswer().substring(1, theoryExam.getSingleStuAnswer().length() - 1).split(",");
            String singleStuAnswer = theoryExam.getSingleStuAnswer();
            String[] singleSplit = singleStuAnswer.split(",");
//            String[] singleSplit = theoryExam.getSingleStuAnswer().split(", ");
            for (String str : singleSplit) {
                if (StringUtils.isBlank(str)) {
                    answerList.add(null);
                } else {
                    answerList.add(str);
                }
            }
        }
        if (StringUtils.isNotBlank(theoryExam.getMultipleStuAnswer())) {
            String s = theoryExam.getMultipleStuAnswer();
            String[] multipleSplit = s.split(",");
//            String[] multipleSplit = theoryExam.getMultipleStuAnswer().split(", ");
            for (String str : multipleSplit) {
                if (StringUtils.isBlank(str)) {
                    answerList.add(null);
                } else {
                    String strSort = sortString(str);
                    answerList.add(strSort);
                }
            }
        }
        if (StringUtils.isNotBlank(theoryExam.getJudgeStuAnswer())) {
            String judgeStuAnswer = theoryExam.getJudgeStuAnswer();
            String[] judgeSplit = judgeStuAnswer.split(",");
            for (String str : judgeSplit) {
                if (StringUtils.isBlank(str)) {
                    answerList.add(null);
                } else {
                    answerList.add(str);
                }
            }
        }
        List<StuTheoryExamDetailDto> detailDtos = new ArrayList<>();
        if (answerList.size() == 0) {
            return null;
        }
        for (int i = 0; i < answerList.size(); i++) {
            if (answerList.get(i) == null) {
                answerList.set(i, "空");
            }

        }
        for (int i = (index - 1) * size; i < (index - 1) * size + size && i < idsList.size() && i < answerList.size(); i++) {
            StuTheoryExamDetailDto detailDto = new StuTheoryExamDetailDto();
            String objectiveQuestionId = idsList.get(i);
            String stuAnswer = answerList.get(i).trim();
            ObjectiveQuestionWithBLOBs objectiveQuestion = objectiveQuestionMapper.selectByPrimaryKey(objectiveQuestionId);
            BigDecimal score = objectiveQuestion.getScore();
            String content = objectiveQuestion.getContent();
            String answer = objectiveQuestion.getAnswer();
            if (stuAnswer.equals("true")) {
                stuAnswer = "正确";
            }
            if (stuAnswer.equals("false")) {
                stuAnswer = "错误";
            }
            if (answer.replaceAll(",", "").equals(stuAnswer)) {
                detailDto.setScore(String.valueOf(score));
                detailDto.setStatus("正确");
            } else {
                detailDto.setScore("0");
                detailDto.setStatus("错误");
            }
            detailDto.setContent(content);
            detailDto.setMychoice(stuAnswer);
            detailDto.setAnswer(answer.replaceAll(",", ""));
            detailDtos.add(detailDto);
        }
        return detailDtos;
    }


    //字母排序
    private static String sortString(String str) {
        char[] charArray = str.toCharArray();
        Arrays.sort(charArray);
        return new String(charArray);
    }

    private List<ObjectiveQuestionWithBLOBs> selectBatchByIdList(List<String> list) {
        ObjectiveQuestionExample example = new ObjectiveQuestionExample();
        example.createCriteria().andObjectiveIdIn(list);
        List<ObjectiveQuestionWithBLOBs> sysObjectiveQuestions = objectiveQuestionMapper.selectByExampleWithBLOBs(example);
        return sysObjectiveQuestions;
    }


    //第一次生成理论考试记录数据  先判断是否生成过理论考试记录数据
    public void GenerateTheoryRecordFirst(String userId) {
        StuTheoryRecord record = theoryRecordMapper.selectByPrimaryKey(userId);
        if (record == null) {
            UserInfo stuUser = userMapper.selectByPrimaryKey(userId);
            StuTheoryRecord stuTheoryRecord = new StuTheoryRecord();
            stuTheoryRecord.setUserId(userId);
            stuTheoryRecord.setSchoolId(stuUser.getSchoolId());
            stuTheoryRecord.setClassId(stuUser.getClassId());
            stuTheoryRecord.setName(stuUser.getName());
            stuTheoryRecord.setStudentId(stuUser.getStudentId());
            stuTheoryRecord.setExamCount(0);
            theoryRecordMapper.insert(stuTheoryRecord);
        }
    }

    //随机生成100分题目
    public List<ObjectiveQuestionWithBLOBs> GenerateTest(String userId, String schoolId, String source) throws IOException {
        List<ObjectiveQuestionWithBLOBs> objectiveQuestionList1 = new ArrayList<>();
        List<ObjectiveQuestionWithBLOBs> objectiveQuestionList2 = new ArrayList<>();
        List<ObjectiveQuestionWithBLOBs> objectiveQuestionList3 = new ArrayList<>();
        if (source.equals("理论考试")) {
            objectiveQuestionList1 = objectiveQuestionMapper.getObjectiveByRandom(0, 35, "普通题库", null, schoolId); //单选
            objectiveQuestionList2 = objectiveQuestionMapper.getObjectiveByRandom(1, 5, "普通题库", null, schoolId);   //多选
            objectiveQuestionList3 = objectiveQuestionMapper.getObjectiveByRandom(2, 10, "普通题库", null, schoolId);  //判断
        } else if (source.equals("理论知识")) {
            objectiveQuestionList1 = objectiveQuestionMapper.getObjectiveByRandom(0, 60, "金融智能", source, schoolId); //单选
            objectiveQuestionList2 = objectiveQuestionMapper.getObjectiveByRandom(1, 30, "金融智能", source, schoolId);   //多选
            objectiveQuestionList3 = objectiveQuestionMapper.getObjectiveByRandom(2, 10, "金融智能", source, schoolId);  //判断
        } else if (source.equals("专业能力")) {
            objectiveQuestionList1 = objectiveQuestionMapper.getObjectiveByRandom(0, 100, "金融智能", source, schoolId); //单选
        } else {
            objectiveQuestionList1 = objectiveQuestionMapper.getObjectiveByRandom(0, 90, "银行从业", source, schoolId); //单选
            objectiveQuestionList2 = objectiveQuestionMapper.getObjectiveByRandom(1, 40, "银行从业", source, schoolId);   //多选
            objectiveQuestionList3 = objectiveQuestionMapper.getObjectiveByRandom(2, 15, "银行从业", source, schoolId);  //判断
        }
        List<ObjectiveQuestionWithBLOBs> objectiveQuestionList = new ArrayList<>();
        objectiveQuestionList.addAll(objectiveQuestionList1);
        objectiveQuestionList.addAll(objectiveQuestionList2);
        objectiveQuestionList.addAll(objectiveQuestionList3);
        List<String> singleIdList = new ArrayList<>();
        List<String> manyIdList = new ArrayList<>();
        List<String> judgeIdList = new ArrayList<>();
        for (int i = 0; i < objectiveQuestionList.size(); i++) {
            ObjectiveQuestionWithBLOBs objectiveQuestion = objectiveQuestionList.get(i);
            String objectiveId = objectiveQuestion.getObjectiveId();
            if (0 == objectiveQuestion.getType()) {
                singleIdList.add(objectiveId);
            } else if (1 == objectiveQuestion.getType()) {
                manyIdList.add(objectiveId);
            } else {
                judgeIdList.add(objectiveId);
            }
        }
        StuTheoryExamWithBLOBs stuTheoryExam = new StuTheoryExamWithBLOBs();
        stuTheoryExam.setTheoryExamId(IdUtil.randomUUID());
        stuTheoryExam.setUserId(userId);
        stuTheoryExam.setExamTime(new Date());
        stuTheoryExam.setExamDuration(120);
        stuTheoryExam.setAnswered(false);
        String singleList = String.valueOf(singleIdList).replace("[", "").replace("]", "");
        stuTheoryExam.setSingleIds(singleList);
        if (!manyIdList.isEmpty()) {
            String manyList = String.valueOf(manyIdList).replace("[", "").replace("]", "");
            stuTheoryExam.setMultipleIds(manyList);
        }
        if (!judgeIdList.isEmpty()) {
            String judgeList = String.valueOf(judgeIdList).replace("[", "").replace("]", "");
            stuTheoryExam.setJudgeIds(judgeList);
        }
        stuTheoryExam.setSource(source);
        theoryExamMapper.insert(stuTheoryExam);
        return objectiveQuestionList;
    }


    @Override
    public List<ObjectiveQuestionWithBLOBs> selectErrors(String userId, String schoolId) {
        StuErrorExample example = new StuErrorExample();
        example.createCriteria().andUserIdEqualTo(userId);
        List<StuErrorWithBLOBs> stuErrors = errorMapper.selectByExampleWithBLOBs(example);

        if (stuErrors.isEmpty()) {
            return null;
        }

        StuErrorWithBLOBs stuError = stuErrors.get(0);
        List<String> allIds = new ArrayList<>();

        allIds.addAll(extractIds(stuError.getSingleIds()));
        allIds.addAll(extractIds(stuError.getMultipleIds()));
        allIds.addAll(extractIds(stuError.getJudgeIds()));

        ObjectiveQuestionExample example1 = new ObjectiveQuestionExample();
        example1.createCriteria().andObjectiveIdIn(allIds);
        example1.setOrderByClause("type");
        return objectiveQuestionMapper.selectByExampleWithBLOBs(example1);
    }

    private List<String> extractIds(String ids) {
        if (ids == null || ids.length() <= 2) {
            return Collections.emptyList();
        }
        return Arrays.asList(ids.split(", "));
    }

//
//    @Override
//    public List<ObjectiveQuestionWithBLOBs> selectErrors(String userId, String schoolId) {
//        StuErrorExample example = new StuErrorExample();
//        example.createCriteria().andUserIdEqualTo(userId);
//        List<StuErrorWithBLOBs> stuErrors = errorMapper.selectByExampleWithBLOBs(example);
//        if (stuErrors.isEmpty()) {
//            return null;
//        } else {
//            List<ObjectiveQuestionWithBLOBs> objectiveQuestionList = new ArrayList<>();
//            StuErrorWithBLOBs stuError = stuErrors.get(0);
//            String singleIds = stuError.getSingleIds();
//            if (!singleIds.isEmpty()) {
//                List<String> singleIdList = Arrays.asList(singleIds.substring(1, singleIds.length() - 1).split(", "));
//                for (int i = 0; i < singleIdList.size(); i++) {
//                    ObjectiveQuestionWithBLOBs objectiveQuestion = objectiveQuestionMapper.selectByPrimaryKey(singleIdList.get(i));
//                    objectiveQuestionList.add(objectiveQuestion);
//                }
//            }
//            String multipleIds = stuError.getMultipleIds();
//            if (!multipleIds.isEmpty()) {
//                List<String> stringList = Arrays.asList(multipleIds.substring(1, multipleIds.length() - 1).split(", "));
//                for (int i = 0; i < stringList.size(); i++) {
//                    ObjectiveQuestionWithBLOBs objectiveQuestion = objectiveQuestionMapper.selectByPrimaryKey(stringList.get(i));
//                    objectiveQuestionList.add(objectiveQuestion);
//                }
//            }
//            String judgeIds = stuError.getJudgeIds();
//            if (!judgeIds.isEmpty()) {
//                List<String> stringList = Arrays.asList(judgeIds.substring(1, judgeIds.length() - 1).split(", "));
//                for (int i = 0; i < stringList.size(); i++) {
//                    ObjectiveQuestionWithBLOBs objectiveQuestion = objectiveQuestionMapper.selectByPrimaryKey(stringList.get(i));
//                    objectiveQuestionList.add(objectiveQuestion);
//                }
//            }
//            return objectiveQuestionList;
//        }
//    }


    @Override
    public Boolean removeError(String userId, String objectiveQuestionId, Integer type) {
        StuErrorExample stuErrorExample = new StuErrorExample();
        stuErrorExample.createCriteria().andUserIdEqualTo(userId);
        List<StuErrorWithBLOBs> stuErrors = errorMapper.selectByExampleWithBLOBs(stuErrorExample);
        if (stuErrors.isEmpty()) {
            return false;
        }
        StuErrorWithBLOBs error = stuErrors.get(0);
        if (0 == type) {
            List<String> stringList = Arrays.asList(error.getSingleIds().substring(1, stuErrors.get(0).getSingleIds().length() - 1).split(", "));
            Set<String> set = new LinkedHashSet<>(stringList);
            set.remove(objectiveQuestionId);
            if (set.isEmpty()) {
                error.setSingleIds(null);
            } else {
                List<String> list = new ArrayList<>(set);
                String singleId = String.valueOf(list).replace("[", "").replace("]", "");
                error.setSingleIds(singleId);
            }
        }
        if (1 == type) {
            Set<String> set = new LinkedHashSet<>(Arrays.asList(error.getMultipleIds().substring(1, error.getMultipleIds().length() - 1).split(", ")));
            set.remove(objectiveQuestionId);
            if (set.isEmpty()) {
                error.setMultipleIds(null);
            } else {
                List<String> list = new ArrayList<>(set);
                String multipleId = String.valueOf(list).replace("[", "").replace("]", "");
                error.setMultipleIds(multipleId);
            }
        }
        if (2 == type) {
            Set<String> set = new LinkedHashSet<>(Arrays.asList(error.getJudgeIds().substring(1, error.getJudgeIds().length() - 1).split(", ")));
            set.remove(objectiveQuestionId);
            if (set.isEmpty()) {
                error.setJudgeIds(null);
            } else {
                List<String> list = new ArrayList<>(set);
                String judgeId = String.valueOf(list).replace("[", "").replace("]", "");
                error.setJudgeIds(judgeId);
            }
        }
        int i = errorMapper.updateByPrimaryKeyWithBLOBs(error);
        return i == 1;
    }
}
