package com.sztzjy.money_management.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;


/**
 * @Author tanzheng
 * @Description 类型转换: Entity - DTO互转
 * @Date 2023/8/9
 * @Param [params]
 **/
@Component
public class ConvertUtil {
    public static final Logger logger = LoggerFactory.getLogger(ConvertUtil.class);


    /**
     * Entity转DTO
     * @param source
     * @param target
     * @return
     * @param <T>
     */
    public  <T> T entityToDTO(Object source, Class<T> target) {
        if (source == null) {
            return null;
        }
        T targetObject = null;
        try {
            targetObject = target.newInstance();
            BeanUtils.copyProperties(source, targetObject);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return targetObject;
    }
 
    /**
     * DTO转Entity
     * @param source
     * @param target
     * @return
     * @param <T>
     */
    public  <T> T DTOToEntity(Object source, Class<T> target) {
        if (source == null) {
            return null;
        }
        T targetObject = null;
        try {
            targetObject = target.newInstance();
            BeanUtils.copyProperties(source, targetObject);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return targetObject;
    }
 
    /**
     * 列表实体转DTO
     * @param sourceList
     * @param target
     * @return
     * @param <T>
     */
    public  <T> List<T> entityToDTOList(Collection<?> sourceList, Class<T> target) {
        if (sourceList == null) {
            return null;
        }
        List<T> targetList = new ArrayList<>(sourceList.size());
 
        try {
            for (Object source : sourceList) {
                T targetObject = target.newInstance();
                BeanUtils.copyProperties(source, targetObject);
                targetList.add(targetObject);
            }
        } catch (Exception e) {
            logger.error("convert error ", e);
        }
        return targetList;
    }
 
    /**
     * DTO转列表实体
     * @param sourceList
     * @param target
     * @return
     * @param <T>
     */
    public  <T> List<T> DTOListToEntity(Collection<?> sourceList, Class<T> target) {
        if (sourceList == null) {
            return null;
        }
        List<T> targetList = new ArrayList<>(sourceList.size());
 
        try {
            for (Object source : sourceList) {
                T targetObject = target.newInstance();
                BeanUtils.copyProperties(source, targetObject);
                targetList.add(targetObject);
            }
        } catch (Exception e) {
            logger.error("convert error ", e);
        }
        return targetList;
    }
 
 
}